/* lsystem2 -- simple lindenmayer system with audio realization of the
	graph using RTcmix WAVETABLE instrument (squares look for
	red lines in them).  "myTimer.C" is the animator object (can't
	use RTtimeit because of graphics context problems)  -- BGG/2003
*/

#include "wx/wxprec.h"

#ifndef WX_PRECOMP
	#include "wx/wx.h"
#endif

#define MAIN		/* So RTcmix globals will be right */
#include <globals.h>

#include "lsystem2.h"
#include <iostream.h>



IMPLEMENT_APP(lsystem2)

bool lsystem2::OnInit()
{
	wxFrame *frame = new wxFrame((wxFrame*) NULL, -1, "lsystem2",
		wxPoint(50, 50), wxSize(1000, 600),
		wxDEFAULT_FRAME_STYLE, "lsystem2");
	frame->Show(TRUE);
	SetTopWindow(frame);

	// this button makes the graph
	wxButton *button1 = new wxButton(frame, BUTTON_HIT1, "realize!",
		wxPoint(100, 550), wxSize(70, 20));

	// this button starts the sound
	wxButton *button2 = new wxButton(frame, BUTTON_HIT2, "go!",
		wxPoint(200, 550), wxSize(70, 20));

	wdc = new wxClientDC(frame);
	recurse_levels = 7;

	RTcmix *rrr = new RTcmix(44100.0, 2, 128);
	sleep(1);
	rrr->cmd("load", 1, "WAVETABLE");
	rrr->cmd("makegen", 7, 1.0, 24.0, 1000.0, 0.0, 1.0, 1.0, 0.0);
	rrr->cmd("makegen", 8, 2.0, 10.0, 1000.0, 1.0, 0.1, 0.3, 0.02, 0.1);
	rrr->cmd("reset", 1, 44100.0);

	theTimer = new myTimer(wdc, rrr);

	return true;
}

void lsystem2::OnButton1(wxCommandEvent& command)
{
	// draw the lsystem
	int x, y, length;

	x = 0;
	y = 250;
	length = 1000/3;

	wdc->SetPen(*wxRED_PEN);
	wdc->DrawLine(x,y, length,y);

	recurse(x+(length/4), y, length-50, recurse_levels-1);
}


void lsystem2::OnButton2(wxCommandEvent& command)
{
	// start the interpretation-timer, at x=0, 100 msecs between 'Notify's
	theTimer->setX(0);
	theTimer->Start(100, wxTIMER_CONTINUOUS);
}

void lsystem2::recurse(int x, int y, int length, int rlevel)
{
	if (rlevel == 0) return;

	wdc->SetPen(*wxRED_PEN);
	wdc->DrawLine(x,y, x+length,y-49);
	wdc->DrawLine(x,y, x+length,y+15);
	wdc->DrawLine(x,y, x+length/2,y+34);

	recurse(x+length, y-49, length-30, rlevel-1);
	recurse(x+length, y+15, length-15, rlevel-1);
	recurse(x+length/2, y+34, length/2, rlevel-1);
}

BEGIN_EVENT_TABLE(lsystem2, wxApp)
	EVT_BUTTON(BUTTON_HIT1, lsystem2::OnButton1)
	EVT_BUTTON(BUTTON_HIT2, lsystem2::OnButton2)
END_EVENT_TABLE()
